@@ -142,24 +142,31 @@ print_stdout(['g++', '--version'], numlines=1)
 print_stdout(['cryptest', 'V'])
 print_stdout(['git', '--version'])
 print_stdout(['openssl', 'version'])
-print_stdout(['darcs', '--version'])
-print_stdout(['darcs', '--exact-version'], label='darcs-exact-version')
 print_stdout(['flappclient', '--version'])
 print_stdout(['valgrind', '--version'])
-print_stdout(['lzip', '--version'])
 
 print_as_ver()
 
 print_setuptools_ver()
 
+print_py_pkg_ver('cffi')
 print_py_pkg_ver('coverage')
-print_py_pkg_ver('trialcoverage')
+print_py_pkg_ver('cryptography')
+print_py_pkg_ver('foolscap')
+print_py_pkg_ver('mock')
+print_py_pkg_ver('Nevow', 'nevow')
+print_py_pkg_ver('pyasn1')
+print_py_pkg_ver('pycparser')
+print_py_pkg_ver('pycrypto', 'Crypto')
+print_py_pkg_ver('pycryptopp')
 print_py_pkg_ver('pyflakes')
-print_py_pkg_ver('zope.interface')
-print_py_pkg_ver('setuptools_darcs')
-print_py_pkg_ver('darcsver')
+print_py_pkg_ver('pyOpenSSL', 'OpenSSL')
+print_py_pkg_ver('simplejson')
+print_py_pkg_ver('six')
+print_py_pkg_ver('trialcoverage')
 print_py_pkg_ver('Twisted', 'twisted')
 print_py_pkg_ver('TwistedCore', 'twisted.python')
 print_py_pkg_ver('TwistedWeb', 'twisted.web')
 print_py_pkg_ver('TwistedConch', 'twisted.conch')
-print_py_pkg_ver('pycryptopp')
+print_py_pkg_ver('zfec')
+print_py_pkg_ver('zope.interface')
