@@ -56,8 +56,7 @@ class DownloadNode:
         self._history = history
         self._download_status = download_status
 
-        k, N = self._verifycap.needed_shares, self._verifycap.total_shares
-        self.share_hash_tree = IncompleteHashTree(N)
+        self.share_hash_tree = IncompleteHashTree(self._verifycap.total_shares)
 
         # we guess the segment size, so Segmentation can pull non-initial
         # segments in a single roundtrip. This populates
@@ -144,6 +143,7 @@ class DownloadNode:
         read_ev = self._download_status.add_read_event(offset, size, now())
         if IDownloadStatusHandlingConsumer.providedBy(consumer):
             consumer.set_download_status_read_event(read_ev)
+            consumer.set_download_status(self._download_status)
 
         lp = log.msg(format="imm Node(%(si)s).read(%(offset)d, %(size)d)",
                      si=base32.b2a(self._verifycap.storage_index)[:8],
@@ -403,6 +403,7 @@ class DownloadNode:
         self._start_new_segment()
 
     def process_blocks(self, segnum, blocks):
+        start = now()
         d = defer.maybeDeferred(self._decode_blocks, segnum, blocks)
         d.addCallback(self._check_ciphertext_hash, segnum)
         def _deliver(result):
@@ -429,6 +430,7 @@ class DownloadNode:
                     seg_ev.activate(when)
                     seg_ev.deliver(when, offset, len(segment), decodetime)
                     eventually(self._deliver, d, c, result)
+            self._download_status.add_misc_event("process_block", start, now())
             self._active_segment = None
             self._start_new_segment()
         d.addBoth(_deliver)
@@ -436,6 +438,7 @@ class DownloadNode:
                      level=log.WEIRD, parent=self._lp, umid="MkEsCg")
 
     def _decode_blocks(self, segnum, blocks):
+        start = now()
         tail = (segnum == self.num_segments-1)
         codec = self._codec
         block_size = self.block_size
@@ -456,7 +459,6 @@ class DownloadNode:
             shares.append(share)
         del blocks
 
-        start = now()
         d = codec.decode(shares, shareids)   # segment
         del shares
         def _process(buffers):
@@ -466,11 +468,13 @@ class DownloadNode:
             del buffers
             if tail:
                 segment = segment[:self.tail_segment_size]
+            self._download_status.add_misc_event("decode", start, now())
             return (segment, decodetime)
         d.addCallback(_process)
         return d
 
     def _check_ciphertext_hash(self, (segment, decodetime), segnum):
+        start = now()
         assert self._active_segment.segnum == segnum
         assert self.segment_size is not None
         offset = segnum * self.segment_size
@@ -478,6 +482,7 @@ class DownloadNode:
         h = hashutil.crypttext_segment_hash(segment)
         try:
             self.ciphertext_hash_tree.set_hashes(leaves={segnum: h})
+            self._download_status.add_misc_event("CThash", start, now())
             return (offset, segment, decodetime)
         except (BadHashError, NotEnoughHashesError):
             format = ("hash failure in ciphertext_hash_tree:"
@@ -508,9 +513,9 @@ class DownloadNode:
                                   if t[0] != segnum]
         return retire
 
-    def _cancel_request(self, c):
+    def _cancel_request(self, cancel):
         self._segment_requests = [t for t in self._segment_requests
-                                  if t[2] != c]
+                                  if t[2] != cancel]
         segnums = [segnum for (segnum,d,c,seg_ev,lp) in self._segment_requests]
         # self._active_segment might be None in rare circumstances, so make
         # sure we tolerate it
