@@ -1,18 +1,12 @@
 
 import os, sys
-from allmydata.scripts.common import BasedirMixin, BaseOptions
+from allmydata.scripts.common import NoDefaultBasedirOptions
 from allmydata.util.assertutil import precondition
 from allmydata.util.encodingutil import listdir_unicode, quote_output
 
-class CreateStatsGathererOptions(BasedirMixin, BaseOptions):
-    default_nodedir = None
 
-    optParameters = [
-        ["node-directory", "d", None, "Specify which directory the stats-gatherer should be created in. [no default]"],
-    ]
-
-    def getSynopsis(self):
-        return "Usage:  %s create-stats-gatherer [options] NODEDIR" % (self.command_name,)
+class CreateStatsGathererOptions(NoDefaultBasedirOptions):
+    subcommand_name = "create-stats-gatherer"
 
 
 stats_gatherer_tac = """
