@@ -1,6 +1,4 @@
-#! /usr/bin/env python
-
-import os, shutil, sys, urllib, time, stat
+import os, shutil, sys, urllib, time, stat, urlparse
 from cStringIO import StringIO
 from twisted.internet import defer, reactor, protocol, error
 from twisted.application import service, internet
@@ -51,10 +49,14 @@ def discardPage(url, stall=False, *args, **kwargs):
     # adapted from twisted.web.client.getPage . We can't just wrap or
     # subclass because it provides no way to override the HTTPClientFactory
     # that it creates.
-    scheme, host, port, path = tw_client._parse(url)
+    scheme, netloc, path, params, query, fragment = urlparse.urlparse(url)
+    assert scheme == 'http'
+    host, port = netloc, 80
+    if ":" in host:
+        host, port = host.split(":")
+        port = int(port)
     factory = StallableDiscardingHTTPClientFactory(url, *args, **kwargs)
     factory.do_stall = stall
-    assert scheme == 'http'
     reactor.connectTCP(host, port, factory)
     return factory.deferred
 
@@ -147,7 +149,7 @@ class SystemFramework(pollmixin.PollMixin):
 
     def tearDown(self, passthrough):
         # the client node will shut down in a few seconds
-        #os.remove(os.path.join(self.clientdir, "suicide_prevention_hotline"))
+        #os.remove(os.path.join(self.clientdir, client.Client.EXIT_TRIGGER_FILE))
         log.msg("shutting down SystemTest services")
         if self.keepalive_file and os.path.exists(self.keepalive_file):
             age = time.time() - os.stat(self.keepalive_file)[stat.ST_MTIME]
@@ -184,9 +186,12 @@ class SystemFramework(pollmixin.PollMixin):
         for i in range(self.numnodes):
             nodedir = os.path.join(self.testdir, "node%d" % i)
             os.mkdir(nodedir)
-            f = open(os.path.join(nodedir, "introducer.furl"), "w")
-            f.write(self.introducer_furl)
-            f.close()
+            f = open(os.path.join(nodedir, "tahoe.cfg"), "w")
+            f.write("[client]\n"
+                    "introducer.furl = %s\n"
+                    "shares.happy = 1\n"
+                    "[storage]\n"
+                    % (self.introducer_furl,))
             # the only tests for which we want the internal nodes to actually
             # retain shares are the ones where somebody's going to download
             # them.
@@ -197,15 +202,12 @@ class SystemFramework(pollmixin.PollMixin):
                 # for these tests, we tell the storage servers to pretend to
                 # accept shares, but really just throw them out, since we're
                 # only testing upload and not download.
-                f = open(os.path.join(nodedir, "debug_no_storage"), "w")
-                f.write("no_storage\n")
-                f.close()
+                f.write("debug_discard = true\n")
             if self.mode in ("receive",):
                 # for this mode, the client-under-test gets all the shares,
                 # so our internal nodes can refuse requests
-                f = open(os.path.join(nodedir, "readonly_storage"), "w")
-                f.write("\n")
-                f.close()
+                f.write("readonly = true\n")
+            f.close()
             c = self.add_service(client.Client(basedir=nodedir))
             self.nodes.append(c)
         # the peers will start running, eventually they will connect to each
@@ -232,33 +234,32 @@ this file are ignored.
         clientdir = self.clientdir = os.path.join(self.testdir, u"client")
         clientdir_str = clientdir.encode(get_filesystem_encoding())
         quiet = StringIO()
-        create_node.create_node(clientdir, {}, out=quiet)
+        create_node.create_node({'basedir': clientdir}, out=quiet)
         log.msg("DONE MAKING CLIENT")
-        f = open(os.path.join(clientdir, "introducer.furl"), "w")
-        f.write(self.introducer_furl + "\n")
-        f.close()
-
+        # now replace tahoe.cfg
         # set webport=0 and then ask the node what port it picked.
-        f = open(os.path.join(clientdir, "webport"), "w")
-        f.write("tcp:0:interface=127.0.0.1\n")
-        f.close()
+        f = open(os.path.join(clientdir, "tahoe.cfg"), "w")
+        f.write("[node]\n"
+                "web.port = tcp:0:interface=127.0.0.1\n"
+                "[client]\n"
+                "introducer.furl = %s\n"
+                "shares.happy = 1\n"
+                "[storage]\n"
+                % (self.introducer_furl,))
 
         if self.mode in ("upload-self", "receive"):
             # accept and store shares, to trigger the memory consumption bugs
             pass
         else:
             # don't accept any shares
-            f = open(os.path.join(clientdir, "readonly_storage"), "w")
-            f.write("true\n")
-            f.close()
+            f.write("readonly = true\n")
             ## also, if we do receive any shares, throw them away
-            #f = open(os.path.join(clientdir, "debug_no_storage"), "w")
-            #f.write("no_storage\n")
-            #f.close()
+            #f.write("debug_discard = true")
         if self.mode == "upload-self":
             pass
+        f.close()
         self.keepalive_file = os.path.join(clientdir,
-                                           "suicide_prevention_hotline")
+                                           client.Client.EXIT_TRIGGER_FILE)
         # now start updating the mtime.
         self.touch_keepalive()
         ts = internet.TimerService(1.0, self.touch_keepalive)
@@ -377,14 +378,9 @@ this file are ignored.
         print
         print "uploading %s" % name
         if self.mode in ("upload", "upload-self"):
-            files[name] = self.create_data(name, size)
-            d = self.control_rref.callRemote("upload_from_file_to_uri",
-                                             files[name], convergence="check-memory convergence string")
-            def _done(uri):
-                os.remove(files[name])
-                del files[name]
-                return uri
-            d.addCallback(_done)
+            d = self.control_rref.callRemote("upload_random_data_from_file",
+                                             size,
+                                             convergence="check-memory")
         elif self.mode == "upload-POST":
             data = "a" * size
             url = "/uri"
@@ -403,8 +399,10 @@ this file are ignored.
             files[name] = self.create_data(name, size)
             u = self.nodes[0].getServiceNamed("uploader")
             d = self.nodes[0].debug_wait_for_client_connections(self.numnodes+1)
-            d.addCallback(lambda res: u.upload(upload.FileName(files[name], convergence="check-memory convergence string")))
-            d.addCallback(lambda results: results.uri)
+            d.addCallback(lambda res:
+                          u.upload(upload.FileName(files[name],
+                                                   convergence="check-memory")))
+            d.addCallback(lambda results: results.get_uri())
         else:
             raise ValueError("unknown mode=%s" % self.mode)
         def _complete(uri):
@@ -421,8 +419,8 @@ this file are ignored.
         uri = uris[name]
 
         if self.mode == "download":
-            d = self.control_rref.callRemote("download_from_uri_to_file",
-                                             uri, "dummy.out")
+            d = self.control_rref.callRemote("download_to_tempfile_and_delete",
+                                             uri)
         elif self.mode == "download-GET":
             url = "/uri/%s" % uri
             d = self.GET_discard(urllib.quote(url), stall=False)
@@ -499,6 +497,13 @@ if __name__ == '__main__':
     mode = "upload"
     if len(sys.argv) > 1:
         mode = sys.argv[1]
+    if sys.maxint == 2147483647:
+        bits = "32"
+    elif sys.maxint == 9223372036854775807:
+        bits = "64"
+    else:
+        bits = "?"
+    print "%s-bit system (sys.maxint=%d)" % (bits, sys.maxint)
     # put the logfile and stats.out in _test_memory/ . These stick around.
     # put the nodes and other files in _test_memory/test/ . These are
     # removed each time we run.
