@@ -277,34 +277,31 @@ class DeepCheckWebGood(DeepCheckBase, unittest.TestCase):
         self.failUnlessEqual(cr.get_storage_index_string(),
                              base32.b2a(n.get_storage_index()), where)
         num_servers = len(self.g.all_servers)
-        needs_rebalancing = bool( num_servers < 10 )
+        self.failUnlessEqual(num_servers, 10, where)
+
+        self.failUnlessEqual(cr.get_happiness(), num_servers, where)
+        self.failUnlessEqual(cr.get_share_counter_good(), num_servers, where)
+        self.failUnlessEqual(cr.get_encoding_needed(), 3, where)
+        self.failUnlessEqual(cr.get_encoding_expected(), num_servers, where)
         if not incomplete:
-            self.failUnlessEqual(cr.needs_rebalancing(), needs_rebalancing,
-                                 str((where, cr, cr.get_data())))
-        d = cr.get_data()
-        self.failUnlessEqual(d["count-shares-good"], 10, where)
-        self.failUnlessEqual(d["count-shares-needed"], 3, where)
-        self.failUnlessEqual(d["count-shares-expected"], 10, where)
+            self.failUnlessEqual(cr.get_host_counter_good_shares(),
+                                 num_servers, where)
+        self.failUnlessEqual(cr.get_corrupt_shares(), [], where)
         if not incomplete:
-            self.failUnlessEqual(d["count-good-share-hosts"], num_servers,
-                                 where)
-        self.failUnlessEqual(d["count-corrupt-shares"], 0, where)
-        self.failUnlessEqual(d["list-corrupt-shares"], [], where)
-        if not incomplete:
-            self.failUnlessEqual(sorted(d["servers-responding"]),
+            self.failUnlessEqual(sorted([s.get_serverid()
+                                         for s in cr.get_servers_responding()]),
                                  sorted(self.g.get_all_serverids()),
                                  where)
-            self.failUnless("sharemap" in d, str((where, d)))
             all_serverids = set()
-            for (shareid, serverids) in d["sharemap"].items():
-                all_serverids.update(serverids)
+            for (shareid, servers) in cr.get_sharemap().items():
+                all_serverids.update([s.get_serverid() for s in servers])
             self.failUnlessEqual(sorted(all_serverids),
                                  sorted(self.g.get_all_serverids()),
                                  where)
 
-        self.failUnlessEqual(d["count-wrong-shares"], 0, where)
-        self.failUnlessEqual(d["count-recoverable-versions"], 1, where)
-        self.failUnlessEqual(d["count-unrecoverable-versions"], 0, where)
+        self.failUnlessEqual(cr.get_share_counter_wrong(), 0, where)
+        self.failUnlessEqual(cr.get_version_counter_recoverable(), 1, where)
+        self.failUnlessEqual(cr.get_version_counter_unrecoverable(), 0, where)
 
 
     def check_and_repair_is_healthy(self, cr, n, where, incomplete=False):
@@ -535,13 +532,13 @@ class DeepCheckWebGood(DeepCheckBase, unittest.TestCase):
         r = data["results"]
         self.failUnlessEqual(r["healthy"], True, where)
         num_servers = len(self.g.all_servers)
-        needs_rebalancing = bool( num_servers < 10 )
-        if not incomplete:
-            self.failUnlessEqual(r["needs-rebalancing"], needs_rebalancing,
-                                 where)
-        self.failUnlessEqual(r["count-shares-good"], 10, where)
+        self.failUnlessEqual(num_servers, 10)
+
+        self.failIfIn("needs-rebalancing", r)
+        self.failUnlessEqual(r["count-happiness"], num_servers, where)
+        self.failUnlessEqual(r["count-shares-good"], num_servers, where)
         self.failUnlessEqual(r["count-shares-needed"], 3, where)
-        self.failUnlessEqual(r["count-shares-expected"], 10, where)
+        self.failUnlessEqual(r["count-shares-expected"], num_servers, where)
         if not incomplete:
             self.failUnlessEqual(r["count-good-share-hosts"], num_servers,
                                  where)
@@ -761,8 +758,8 @@ class DeepCheckWebGood(DeepCheckBase, unittest.TestCase):
 
     def do_cli_manifest_stream1(self):
         basedir = self.get_clientdir(0)
-        d = self._run_cli(["manifest",
-                           "--node-directory", basedir,
+        d = self._run_cli(["--node-directory", basedir,
+                           "manifest",
                            self.root_uri])
         def _check((out,err)):
             self.failUnlessEqual(err, "")
@@ -789,8 +786,8 @@ class DeepCheckWebGood(DeepCheckBase, unittest.TestCase):
 
     def do_cli_manifest_stream2(self):
         basedir = self.get_clientdir(0)
-        d = self._run_cli(["manifest",
-                           "--node-directory", basedir,
+        d = self._run_cli(["--node-directory", basedir,
+                           "manifest",
                            "--raw",
                            self.root_uri])
         def _check((out,err)):
@@ -802,8 +799,8 @@ class DeepCheckWebGood(DeepCheckBase, unittest.TestCase):
 
     def do_cli_manifest_stream3(self):
         basedir = self.get_clientdir(0)
-        d = self._run_cli(["manifest",
-                           "--node-directory", basedir,
+        d = self._run_cli(["--node-directory", basedir,
+                           "manifest",
                            "--storage-index",
                            self.root_uri])
         def _check((out,err)):
@@ -814,8 +811,8 @@ class DeepCheckWebGood(DeepCheckBase, unittest.TestCase):
 
     def do_cli_manifest_stream4(self):
         basedir = self.get_clientdir(0)
-        d = self._run_cli(["manifest",
-                           "--node-directory", basedir,
+        d = self._run_cli(["--node-directory", basedir,
+                           "manifest",
                            "--verify-cap",
                            self.root_uri])
         def _check((out,err)):
@@ -830,8 +827,8 @@ class DeepCheckWebGood(DeepCheckBase, unittest.TestCase):
 
     def do_cli_manifest_stream5(self):
         basedir = self.get_clientdir(0)
-        d = self._run_cli(["manifest",
-                           "--node-directory", basedir,
+        d = self._run_cli(["--node-directory", basedir,
+                           "manifest",
                            "--repair-cap",
                            self.root_uri])
         def _check((out,err)):
@@ -846,8 +843,8 @@ class DeepCheckWebGood(DeepCheckBase, unittest.TestCase):
 
     def do_cli_stats1(self):
         basedir = self.get_clientdir(0)
-        d = self._run_cli(["stats",
-                           "--node-directory", basedir,
+        d = self._run_cli(["--node-directory", basedir,
+                           "stats",
                            self.root_uri])
         def _check3((out,err)):
             lines = [l.strip() for l in out.split("\n") if l]
@@ -866,8 +863,8 @@ class DeepCheckWebGood(DeepCheckBase, unittest.TestCase):
 
     def do_cli_stats2(self):
         basedir = self.get_clientdir(0)
-        d = self._run_cli(["stats",
-                           "--node-directory", basedir,
+        d = self._run_cli(["--node-directory", basedir,
+                           "stats",
                            "--raw",
                            self.root_uri])
         def _check4((out,err)):
@@ -1010,9 +1007,8 @@ class DeepCheckWebBad(DeepCheckBase, unittest.TestCase):
             self.failUnless(ICheckResults.providedBy(cr), (cr, type(cr), where))
             self.failUnless(cr.is_healthy(), (cr.get_report(), cr.is_healthy(), cr.get_summary(), where))
             self.failUnless(cr.is_recoverable(), where)
-            d = cr.get_data()
-            self.failUnlessEqual(d["count-recoverable-versions"], 1, where)
-            self.failUnlessEqual(d["count-unrecoverable-versions"], 0, where)
+            self.failUnlessEqual(cr.get_version_counter_recoverable(), 1, where)
+            self.failUnlessEqual(cr.get_version_counter_unrecoverable(), 0, where)
             return cr
         except Exception, le:
             le.args = tuple(le.args + (where,))
@@ -1022,31 +1018,28 @@ class DeepCheckWebBad(DeepCheckBase, unittest.TestCase):
         self.failUnless(ICheckResults.providedBy(cr), where)
         self.failIf(cr.is_healthy(), where)
         self.failUnless(cr.is_recoverable(), where)
-        d = cr.get_data()
-        self.failUnlessEqual(d["count-recoverable-versions"], 1, where)
-        self.failUnlessEqual(d["count-unrecoverable-versions"], 0, where)
+        self.failUnlessEqual(cr.get_version_counter_recoverable(), 1, where)
+        self.failUnlessEqual(cr.get_version_counter_unrecoverable(), 0, where)
         return cr
 
     def check_has_corrupt_shares(self, cr, where):
         # by "corrupt-shares" we mean the file is still recoverable
         self.failUnless(ICheckResults.providedBy(cr), where)
-        d = cr.get_data()
         self.failIf(cr.is_healthy(), (where, cr))
         self.failUnless(cr.is_recoverable(), where)
-        d = cr.get_data()
-        self.failUnless(d["count-shares-good"] < 10, where)
-        self.failUnless(d["count-corrupt-shares"], where)
-        self.failUnless(d["list-corrupt-shares"], where)
+        self.failUnless(cr.get_share_counter_good() < 10, where)
+        self.failUnless(cr.get_corrupt_shares(), where)
         return cr
 
     def check_is_unrecoverable(self, cr, where):
         self.failUnless(ICheckResults.providedBy(cr), where)
-        d = cr.get_data()
         self.failIf(cr.is_healthy(), where)
         self.failIf(cr.is_recoverable(), where)
-        self.failUnless(d["count-shares-good"] < d["count-shares-needed"], (d["count-shares-good"], d["count-shares-needed"], where))
-        self.failUnlessEqual(d["count-recoverable-versions"], 0, where)
-        self.failUnlessEqual(d["count-unrecoverable-versions"], 1, where)
+        self.failUnless(cr.get_share_counter_good() < cr.get_encoding_needed(),
+                        (cr.get_share_counter_good(), cr.get_encoding_needed(),
+                         where))
+        self.failUnlessEqual(cr.get_version_counter_recoverable(), 0, where)
+        self.failUnlessEqual(cr.get_version_counter_unrecoverable(), 1, where)
         return cr
 
     def do_check(self, ignored):
